/**
 * @file MessageIDs.h
 *
 * Declaration of ids for debug messages.
 *
 * @author Martin Lötzsch
 */

#pragma once

#include "Tools/Enum.h"

/**
 * IDs for debug messages
 *
 * To distinguish debug messages, they all have an id.
 */
ENUM(MessageID,
{,
  undefined,
  idProcessBegin,
  idProcessFinished,

  idActivationGraph,
  idAlternativeRobotPoseHypothesis,
  idAnnotation,
  idAudioData,
  idBallModel,
  idBallPercept,
  idBallSpots,
  idBehaviorMotionRequest,
  idBehaviorStatus,
  idBodyContour,
  idCameraInfo,
  idCameraMatrix,
  idColorCalibration,
  idDropInPlayer,
  idExternStrategyInput,
  idFieldBoundary,
  idFootGroundContactState,
  idFsrSensorData,
  idFrameInfo,
  idGameInfo,
  idGoalPercept,
  idGroundTruthOdometryData,
  idGroundTruthWorldState,
  idImage,
  idImageCoordinateSystem,
  idInertialData,
  idInertialSensorData,
  idJointAngles,
  idJointDataPrediction,
  idJointSensorData,
  idJointVelocities,
  idJPEGImage,
  idKeyStates,
  idKickoff,
  idKickPose,
  idLinePercept,
  idLineSpots,
  idLocalizationTeamBall,
  idLowFrameRateImage,
  idMotionInfo,
  idMotionRequest,
  idNetworkThumbnail,
  idObstacleModel,
  idObstacleModelCompressed,
  idOdometer,
  idOdometryData,
  idOpponentTeamInfo,
  idOwnTeamInfo,
  idPenaltyMarkPercept,
  idPlayersPercept,
  idRealisticBallPercepts,
  idReceivedSPLStandardMessages,
  idRobotHealth,
  idRobotInfo,
  idRobotPose,
  idScanlineRegions,
  idSelfLocalizationHypotheses,
  idSideConfidence,
  idSPLStandardBehaviorStatus,
  idStopwatch,
  idSystemSensorData,
  idTeamBallModel,
  idTeammateData,
  idTeammateReliability,
  idTeamPlayersModel,
  idThrowInBall,
  idThumbnail,
  idUsSensorData,
  idZmp,

  numOfDataMessageIDs, /**< everything below this does not belong into log files */

  // ids used in team communication
  idFieldFeatureOverview = numOfDataMessageIDs,
  idNTPHeader,
  idNTPIdentifier,
  idNTPRequest,
  idNTPResponse,
  idRobot,
  idTeam,
  idTeammateBallAge,
  idTeammateHasGroundContact,
  idTeammateIsPenalized,
  idTeammateIsUpright,
  idTeammateRoles,
  idTeammateTimeOfLastGroundContact,
  idTeammateFieldCoverage,
  idWhistle,

  // infrastructure
  idConsole,
  idDebugDataChangeRequest,
  idDebugDataResponse,
  idDebugDrawing,
  idDebugDrawing3D,
  idDebugImage,
  idDebugJPEGImage,
  idDebugRequest,
  idDebugResponse,
  idDrawingManager,
  idDrawingManager3D,
  idJointCalibration,
  idJointRequest,
  idLogResponse,
  idModuleRequest,
  idModuleTable,
  idMotionNet,
  idPlot,
  idQueueFillRequest,
  idRobotDimensions,
  idRobotname,
  idStreamSpecification,
  idText,
  idUSRequest,
  idWalkingEngineKick,
  idTestImagePath,
});
